cluster<-TRUE
test<-TRUE
###############
## Libraries ##
###############

library(ineq)

###############
## Locations ##
###############

if(cluster){
  setwd("hpc_work")
  setwd("SV")
}else{
  setwd("C:\\Users\\Sebastien\\Dropbox\\My_Research\\filibuster (1)\\")
}

if (cluster & !test){
  prefix_pgm<-"pgm/"
  prefix_tbl<-"tbl/"
  prefix_gph<-"gph/"
}

if (cluster & test){
  prefix_pgm<-"pgm/"
  prefix_tbl<-"tbl_test/"
  prefix_gph<-"gph/"
}

if (!cluster & test){
  prefix_pgm<-"pgm\\"
  prefix_tbl<-"tbl_sim\\tests\\"
  prefix_gph<-"gph\\tests\\"
}

if (!cluster & !test){
  prefix_pgm<-"pgm\\"
  prefix_tbl<-"tbl_sim\\"
  prefix_gph<-"gph\\"
}
prefix_tbl<-"tbl_test/agenda_4nomins_intercor/"


################
## Parameters ##
################
# Random seed
set.seed(2002)

# Number of nominations
nvotes<-4

# Number of simulations
ns<-200

# Group sizes
size_maj<-55
size_min<-45
size_senate<-100

#Rules used
rules_suff<-paste0("allrules",nvotes)

# Functions related to the rules of thumb, e.g. applying them to a vector of intensities
source(paste0(prefix_pgm,"rules_functions_agenda.R"))

rules_used<-combinations_of_votes(nvotes)
rules_used2<-combinations_of_votes(nvotes)

numrules<-nrow(rules_used)
minrules<-seq(1:numrules)
majrules<-seq(1:numrules)
minrulesnames<-rev(paste0("conc",minrules))
majrulesnames<-rev(paste0("conc",minrules))

# 
# if (rules_suff=="giniquart"){
#   minrules<-seq(0,1,length=5)
#   majrules<-seq(0,1,length=5)
#   minrulesnames<-c("conc 0","Q1","Med","Q3","full conc")
#   majrulesnames<-c("conc 0","Q1","Med","Q3","full conc") 
# }
# if (rules_suff=="straus"){
#   minrules<-c(0,.05,.7,.33,.2)
#   majrules<-c(0,.05,.7,.33,.2)
#   minrulesnames<-c("ruleone","rules","ruletop","rulea","rulec")
#   majrulesnames<-c("ruleone","rules","ruletop","rulea","rulec")
# }
# if(rules_suff=="giniquint"){
#   minrules<-seq(0,1,length=6)
#   majrules<-seq(0,1,length=6)
#   minrulesnames<-c("conc 0","P20","P40","P60","P80","full conc")
#   majrulesnames<-c("conc 0","P20","P40","P60","P80","full conc")
# }
# if(rules_suff=="allrules5"){
#   minrules<-c(0,0.2,0.33,0.5,0.6,0.8,1)
#   majrules<-c(0,0.2,0.33,0.5,0.6,0.8,1)
#   minrulesnames<-c("conc0","conc1","conc2","conc3","conc4","conc5","conc6")
#   majrulesnames<-c("conc0","conc1","conc2","conc3","conc4","conc5","conc6")
# }


# Number of strategies
numstratmaj<-length(majrules)
numstratmin<-length(minrules)

# Agenda name
agendaname<-"all_nomin_withblocks"

# Storing equilibria
equilibria<-c()

# Parameters to find the normal correlation for our correlated beta
precision<-.01 # gap between normal correlations to look at when checking which
#one we should use
numsims<-1000 # Number of draws to do when looking at the beta correlation
# generated by a normal correlation.

# Linear correlations
targetcor_intra<-c(0.5)
targetcor_inter<-c(0.3)

###############
## Functions ##
###############



# Functions to compute the nash equil of a normal form game
source(paste0(prefix_pgm,"finding_nash_equil_normalformgame_cluster.R"))

# Functions to transform uniform correlations in to normal correlations, and create
# the normal correlation matrix
source(paste0(prefix_pgm,"correlation_functions.R"))

# Extra function to generate the  normal correlations for the 
# nomination types we care about
source(paste0(prefix_pgm,"correlation_agend.R"))

# Functions to generate intensities with given properties
source(paste0(prefix_pgm,"intensities_generation.R"))

# Functions to do operations on  the final matrix of intensities
source(paste0(prefix_pgm,"welfare_calc.R"))

##################################################
## Build the slates and the normal correlations ##
##################################################
####
## Build Index of Possible Distributions
####
majdists<-matrix(c(5,2,1,2,5,1),ncol=2)
#majdists<-matrix(c(5,2,1,2),ncol=2)


# first threes are alphas, last threes are betas.
mindists<-matrix(c(5,2,1,2,5,1),ncol=2)
#mindists<-matrix(c(5,2,1,2),ncol=2)
num_majdists<-length(majdists[,1])
num_mindists<-length(mindists[,1])
x<-as.matrix(expand.grid(rep(list(1:(num_majdists*num_mindists)), nvotes))) 

## Combinations of potential slates

possible_lists<-c()
for(line in 1:nrow(x)){
  if(!is.unsorted(x[line,])){
    possible_lists<-rbind(possible_lists,x[line,])
  }
}
combs.num<-nrow(possible_lists)

# Possible type of nominees (given by one maj dist and one min dist)

combs<-possible_lists
save(combs,file=paste0(prefix_tbl,"slate_indices",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))

# Generate index for specific distributions

combs.mat<-matrix(NA,num_majdists*num_mindists,7)
colnames(combs.mat)<-c("index","majdist","mindist","amaj","bmaj","amin","bmin")
combs.mat[,1]<-c(1:(num_majdists*num_mindists))
combs.mat[,2:3]<-as.matrix(expand.grid(c(1:num_majdists),c(1:num_mindists)))
for(i in 1:(num_majdists*num_mindists)){
  combs.mat[i,4]<-majdists[combs.mat[i,2],1]
  combs.mat[i,5]<-majdists[combs.mat[i,2],2]
  combs.mat[i,6]<-mindists[combs.mat[i,3],1]
  combs.mat[i,7]<-mindists[combs.mat[i,3],2]
}
save(combs.mat,file=paste0(prefix_tbl,"slate_indices_to_distribution",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))

# Add the required correlations
targetcors<-unique(c(targetcor_intra,targetcor_inter))
l<-length(targetcors)



arrayofcorstypes<-array(NA,dim=c(nrow(combs.mat),4,l))
for(cor in targetcors){
  ind<-which(targetcors==cor)
  thefcors<-getmethefcors(combs.mat,precision,numsims,cor)
  tothearray<-cbind(thefcors[,1],thefcors[,8:10])
  arrayofcorstypes[,,ind]<-tothearray
write.csv(tothearray,file=paste0(prefix_tbl,"normalcorrelations_targetcoris",ind,".csv"))
}


# arrayofcorstypes contains normal correlations to use to get a given 
# correlation between senators


##########
## LOOP ##
##########

# For a desired correlation profile
for(intracor in targetcor_intra){#intra party cor
  for(intercor in targetcor_inter){ # interparty cor
    
    # Storing payoff matrix
    payoffmats_maj<-array(NA,dim=c(combs.num,numstratmaj,numstratmaj))
    payoffmats_min<-array(NA,dim=c(combs.num,numstratmin,numstratmin))
    
    # For one typical slate
    for(slate_index in 1:combs.num){

      #Matrices to store the results for this slate
      # Minority welfare
      minmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
      rownames(minmatrix)<-minrulesnames
      colnames(minmatrix)<-majrulesnames
      
      # Majority welfare
      majmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
      rownames(majmatrix)<-minrulesnames
      colnames(majmatrix)<-majrulesnames
      
      # Minority blocks
      blockmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
      rownames(blockmatrix)<-minrulesnames
      colnames(blockmatrix)<-majrulesnames
      
      # minority blocks per nominee
      blockarray<-array(NA,dim=c(numstratmin,numstratmaj,nvotes))
      rownames(blockarray)<-minrulesnames
      colnames(blockarray)<-majrulesnames
      
      # Share of efficiency achieved
      share_eff_welfarematrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
      rownames(share_eff_welfarematrix)<-minrulesnames
      colnames(share_eff_welfarematrix)<-majrulesnames
      
      # For one slate of nominee types, draw the intensities:
      slate_types<-combs[slate_index,]
      matrixval_maj<-c()
      matrixval_min<-c()
      possible_cor_and_type<-1
  
      # One nomination...
      for(nomination in 1:nvotes){
        
        # Get the slate's properties
        nomin_type<-combs.mat[slate_types[nomination],]
      
        # Find the normal correlations we want in arrayofcorstypes
        index_cor_maj<-which(targetcors==intracor)
        index_cor_min<-which(targetcors==intracor)
        index_cor_inter<-which(targetcors==intercor)
        
        intra_maj<-arrayofcorstypes[nomin_type[1],2,index_cor_maj]
        intra_min<-arrayofcorstypes[nomin_type[1],3,index_cor_min]
        inter<-arrayofcorstypes[nomin_type[1],4,index_cor_inter]
        
        # Create covariance matrix. Is it symmetric def pos?
        Sig<-create_sigma(intra_maj,intra_min,inter,size_senate,size_maj)
        evalues_sigma<-eigen(Sig, symmetric=TRUE,only.values=TRUE)
        possible_profile<-!any(evalues_sigma$values<=0)
       
        # If it is, generate the intensities
        if(possible_profile){
          matrixval_all<-int_beta3_f_withcor(nomin_type[4],nomin_type[5],
                                             nomin_type[6],nomin_type[7],
                                             Sig,ns,size_senate,size_maj)
          matrixval_maj<-cbind(matrixval_maj,matrixval_all$drawmaj)
          matrixval_min<-cbind(matrixval_min,matrixval_all$drawmin)
        }else{possible_cor_and_type<-0}
      }
      
      if(possible_cor_and_type==1){
        
        # compute outcomes under filibuster and majority rule
        majintens<-tot_intensities(matrixval_maj)
        minintens<-tot_intensities(matrixval_min)
        
        eff_outcome<-sign(majintens-minintens)
        eff_outcome2<-(eff_outcome+1)/2
        
        efficient_welfare<-rowSums(majintens*eff_outcome2)+rowSums(minintens*(1-eff_outcome2))
        maj_vote_welfare<-rowSums(majintens)
        filib_welfare<-rowSums(minintens)
        share_eff_welfare_majvote<-mean((maj_vote_welfare+efficient_welfare)/(2*efficient_welfare)*100)
        share_eff_welfare_filib<-mean((filib_welfare+efficient_welfare)/(2*efficient_welfare)*100)
        
        # Commpute outcomes for all possible minority/majority rule correlation
        for (quantile_maj in majrules){
          for(quantile_min in minrules){
            
            majindex<-which(majrules==quantile_maj)
            minindex<-which(minrules==quantile_min)
            
            # Rule used
            rule_maj<-rules_used2[majindex,]
            rule_min<-rules_used2[minindex,]
            majres<-apply_chosen_rule(matrixval_maj,rule=rule_maj)
            minres<-apply_chosen_rule(matrixval_min,rule=rule_min)
            
            maj_votes<-majres$matrixofvotes
            maj_intens<-majres$matrixofintens
            min_votes<-minres$matrixofvotes
            min_intens<-minres$matrixofintens
            
            welf<-welfare_rule(maj_intens,min_intens,maj_votes,min_votes)
            welf<-unlist(lapply(welf,mean))
            
            # save values
            
            # Majority welfare
            majmatrix[minindex,majindex]<-welf[1]
            # Minority welfare
            minmatrix[minindex,majindex]<-welf[2]
            # Share of efficiency achieved
            share_eff_welfarematrix[minindex,majindex]<-welf[5]
            # number of blocks
            blockmatrix[minindex,majindex]<-welf[6]
            # number of blocks for specific nominee
            blockarray[minindex,majindex,]<-welf[7:(6+nvotes)]
            
            
          }
        }
        
        
        
        payoffmats_maj[slate_index,,]<-majmatrix
        payoffmats_min[slate_index,,]<-minmatrix
        majpayoffs<-t(majmatrix)
        minpayoffs<-t(minmatrix)
        blockmat<-t(blockmatrix)
        effmat<-t(share_eff_welfarematrix)
        
        
        # Find equilibrium of the normal form game
        
        # Find Nash
        allnasheq<-allnasheq33(majpayoffs,minpayoffs)
        #Output the fully mix nashes
        fullmixnashes<-allnasheq$fullmix
        lfullmix<-length(fullmixnashes[[1]])
        nfullmix<-length(fullmixnashes)
        
        if(lfullmix!=0){
          for(fullmixequil in 1:nfullmix){
            maj_strat<-fullmixnashes[[fullmixequil]][1,]
            min_strat<-fullmixnashes[[fullmixequil]][2,]
            
            # proba of each case
            probapay<-maj_strat%*%t(min_strat)
            
            #payoffs
            maj_pay<-sum(majpayoffs*probapay)
            min_pay<-sum(minpayoffs*probapay)
            
            #blocks
            expectedblocks<-sum(blockmat*probapay)
            
            #blocks by nominee
            expectedblocks_disagg<-c()
            for(nominee in 1:nvotes){
              bmatrix<-t(blockarray[,,nominee])
              expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
            }
            
            
            #efficiency share
            expectedeffshare<-sum(effmat*probapay)
            
            #save
            newequilibrium<-c(slate_index,intracor,intracor,intercor,maj_strat,min_strat,
                              maj_pay,min_pay,expectedblocks,expectedeffshare,
                              share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
            equilibria<-rbind(equilibria,newequilibrium)
          }
        }
        
        # Pure Nash
        purenashes<-allnasheq$pure
        lpure<-length(purenashes[[1]])
        numpure<-length(purenashes)
        if(lpure!=0){
          for(purequil in 1:numpure){
            
            maj_strat<-purenashes[[purequil]][1,]
            min_strat<-purenashes[[purequil]][2,]
            
            # proba of each case
            probapay<-maj_strat%*%t(min_strat)
            
            #payoffs
            maj_pay<-sum(majpayoffs*probapay)
            min_pay<-sum(minpayoffs*probapay)
            
            #blocks
            expectedblocks<-sum(blockmat*probapay)
            
            #blocks by nominee
            expectedblocks_disagg<-c()
            for(nominee in 1:nvotes){
              bmatrix<-t(blockarray[,,nominee])
              expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
            }
            
            
            #efficiency share
            expectedeffshare<-sum(effmat*probapay)
            #save
            newequilibrium<-c(slate_index,intracor,intracor,intercor,maj_strat,min_strat,
                              maj_pay,min_pay,expectedblocks,expectedeffshare,
                              share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
            equilibria<-rbind(equilibria,newequilibrium)
          }
        }
        
        
        # Partial mix nash
        partmixnashes<-allnasheq$partmix
        lpartmix<-length(partmixnashes[[1]])
        numpartmix<-length(partmixnashes)
        if(lpartmix!=0){
          for(partmixequil in 1:numpartmix){
            
            maj_strat<-partmixnashes[[partmixequil]][1,]
            min_strat<-partmixnashes[[partmixequil]][2,]
            
            # proba of each case
            probapay<-maj_strat%*%t(min_strat)
            
            #payoffs
            maj_pay<-sum(majpayoffs*probapay)
            min_pay<-sum(minpayoffs*probapay)
            
            #blocks
            expectedblocks<-sum(blockmat*probapay)
            
            #blocks by nominee
            expectedblocks_disagg<-c()
            for(nominee in 1:nvotes){
              bmatrix<-t(blockarray[,,nominee])
              expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
            }
            
            
            #efficiency share
            expectedeffshare<-sum(effmat*probapay)
            #save
            newequilibrium<-c(slate_index,intracor,intracor,intercor,maj_strat,min_strat,
                              maj_pay,min_pay,expectedblocks,expectedeffshare,
                              share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
            equilibria<-rbind(equilibria,newequilibrium)
          }
        }
      }
    }
  }
}
# Put all of this in a beautiful table
equilibria<-data.frame(equilibria)
names_indiv_block<-c()
for(nominee in 1:nvotes){
  names_indiv_block<-c(names_indiv_block,paste0("block_on_nominee_",nominee))
}
names(equilibria)<-c("slate index","cormaj","cormin","intercor",majrulesnames,minrulesnames,
                     "majwelfare","minwelfare","blocks","effshare","effshare_majvoting",
                     "effshare_filib",names_indiv_block)
head(equilibria)
save(equilibria,file=paste0(prefix_tbl,"equilibria",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))
write.csv(equilibria,file=paste0(prefix_tbl,"equilibria",rules_suff,"agenda",agendaname,"numnomin",nvotes,".csv"))
save(payoffmats_maj,file=paste0(prefix_tbl,"payoffmats_maj",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))
save(payoffmats_min,file=paste0(prefix_tbl,"payoffmats_min",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))