########################################################################
##############################  Rules  #################################
########################################################################
library(ineq)


## Create all possible combination of votes (i.e. all possible rules)

# Create all the possible combinations of x votes on n nominations, x<=n
combinations_of_votes<-function(n){
  
  pastlist<-matrix(c(0,1),nrow=2)
  numvotes<-0
  for (step in 1:(n-1)){
    numvotes<-step+1
    newlist<-c()
    numelinpastlist<-nrow(pastlist)
    for (el in 1:numelinpastlist){
      pastlist[2,]
      s<-sum(pastlist[el,])
      m<-max(pastlist[el,])
      if (numvotes-s>=m){
        for(new in m:(numvotes-s)){
          newel<-c(pastlist[el,],new)
          newlist<-rbind(newlist,newel)
        }
      }
    }
    pastlist<-newlist
  }
  
  newlist[which(rowSums(newlist)==n),]
}

all_rules<-combinations_of_votes(nvotes)

# Function to select rule with the right concentration quantile
#num nominations: number of nominations
# quantile_concentration: from the gini, which quantile of concentration we
# want 
choosen_rule<-function(numnominations,quantile_concentration){
  n<-numnominations
  q<-quantile_concentration
 
  #all_rules<-combinations_of_votes(n)
  
  # Rank concentration level of each rule
  
  # Apply inequality measure
  rankmeasures<-c()
  rankmat<-c()
  measures<-c("Gini")
              #, "RS", "Atkinson", "Theil", "Kolm", "var",
              #"square.var", "entropy")
  all_rules_temp<-all_rules
  for(measure in measures){
    rankmeasures<-c(rankmeasures,paste0("rank",measure))
    ineqvec<-apply(all_rules_temp,1,FUN=ineq,parameter=NULL,type=measure)
    ineqvec<-unlist(ineqvec)
    rankineqvec<-rank(ineqvec)
    all_rules_temp<-cbind(all_rules_temp,ineqvec)
    rankmat<-cbind(rankmat,rankineqvec)
  }
  all_rules_temp<-data.frame(cbind(all_rules_temp,rankmat),row.names=NULL)
  names(all_rules_temp)[(n+1):(n+length(measures))]<-measures
  names(all_rules_temp)[(n+1+length(measures)):ncol(all_rules_temp)]<-rankmeasures

  
  temp<-abs(all_rules_temp$Gini-quantile(all_rules_temp$Gini,probs=q))
  unlist(all_rules_temp[which(temp==min(temp)),][1,][1:n])
  
}


# x <- rmvnorm(n=500, mean=c(1,2), sigma=sigma, method="chol")

# for given rule as defined by a distribution of votes
# from lowest to highest intensity, apply to a vector
# of intensities
# Input: vector of intensities, rule  (vector of votes)
# Output: vector of votes

mrule<-function(intensities,rule){
  votes<-rep(0,length(intensities))
  rank_intens<-rank(intensities)
  votes[order(rank_intens)]<-rule
  votes
}


# For one party, generate matrix of votes over nominations
# given the rule. Also generate total welfare

# Input: Matrix of intensities, rule (as above)
# 

apply_chosen_rule<-function(matrixval,rule){
  party_size<-nrow(matrixval)/ns
  numvotes<-ncol(matrixval)
  # matrix with party vote total
  matrixofvotes<-matrix(NA,nrow=ns,ncol=numvotes)
  # matrix with party intensity total
  matrixofintens<-matrix(NA,nrow=ns,ncol=numvotes)
  
  # convert into votes: generate a (ns*n)*nvotes matrix
  votes<-t(apply(matrixval,1,mrule,rule=rule))
  
  #generate simulation index
  id<-rep(1:ns,party_size)
  
  # Sum number of votes (for outcome) and intensities (for welfare)
  for (i in 1:nvotes){
    ## tapply(Summary Variable, Group Variable, Function)
    # Sum votes within party on each issue by simulation id
    votes2<-tapply(votes[,i],id,sum)
    # Same with intensities
    
    intensities2<-tapply(matrixval[,i],id,sum)
    matrixofvotes[,i]<-votes2
    matrixofintens[,i]<-intensities2
  }
  # MATRIX OF VOTES: sum number of votes for one party
  
  out<-list(matrixofvotes,matrixofintens)
  names(out)<-c("matrixofvotes","matrixofintens")
  out
}


