cluster<-TRUE
test<-TRUE
###############
## Libraries ##
###############

library(ineq)

###############
## Locations ##
###############

if(cluster){
  setwd("hpc_work")
  setwd("SV")
}else{
  setwd("C:\\Users\\Sebastien\\Dropbox\\My_Research\\filibuster (1)\\")
}

if (cluster & !test){
  prefix_pgm<-"pgm/"
  prefix_tbl<-"tbl/"
  prefix_gph<-"gph/"
}

if (cluster & test){
  prefix_pgm<-"pgm/"
  prefix_tbl<-"tbl_test/"
  prefix_gph<-"gph/"
}

if (!cluster & test){
  prefix_pgm<-"pgm\\"
  prefix_tbl<-"tbl_sim\\tests\\"
  prefix_gph<-"gph\\tests\\"
}

if (!cluster & !test){
  prefix_pgm<-"pgm\\"
  prefix_tbl<-"tbl_sim\\"
  prefix_gph<-"gph\\"
}


################
## Parameters ##
################
# Random seed
set.seed(2002)

# Number of nominations
nvotes<-5

# Number of simulations
ns<-100

# Group sizes
size_maj<-55
size_min<-45
size_senate<-100

#Rules used
rules_suff<-"giniquart"

if (rules_suff=="giniquart"){
  minrules<-seq(0,1,length=5)
  majrules<-seq(0,1,length=5)
  minrulesnames<-c("conc 0","Q1","Med","Q3","full conc")
  majrulesnames<-c("conc 0","Q1","Med","Q3","full conc") 
}
if (rules_suff=="straus"){
  minrules<-c(0,.05,.7,.33,.2)
  majrules<-c(0,.05,.7,.33,.2)
  minrulesnames<-c("ruleone","rules","ruletop","rulea","rulec")
  majrulesnames<-c("ruleone","rules","ruletop","rulea","rulec")
}
if(rules_suff=="giniquint"){
  minrules<-seq(0,1,length=6)
  majrules<-seq(0,1,length=6)
  minrulesnames<-c("conc 0","P20","P40","P60","P80","full conc")
  majrulesnames<-c("conc 0","P20","P40","P60","P80","full conc")
}
if(rules_suff=="allrules5"){
  minrules<-c(0,0.2,0.33,0.5,0.6,0.8,1)
  majrules<-c(0,0.2,0.33,0.5,0.6,0.8,1)
  minrulesnames<-c("conc0","conc1","conc2","conc3","conc4","conc5","conc6")
  majrulesnames<-c("conc0","conc1","conc2","conc3","conc4","conc5","conc6")
}


# Number of strategies
numstratmaj<-length(majrules)
numstratmin<-length(minrules)

# Agenda name
agendaname<-"all_nomin_withblocks"

# Storing equilibria
equilibria<-c()



###############
## Functions ##
###############
source(paste0(prefix_pgm,"rules_functions_agenda.R"))
source(paste0(prefix_pgm,"finding_nash_equil_normalformgame_cluster.R"))



############  Build Index of Possible Distributions  ###########
majdists<-matrix(c(5,2,1,2,5,1),ncol=2)
# first threes are alphas, last threes are betas.
mindists<-matrix(c(5,2,1,2,5,1),ncol=2)
num_majdists<-length(majdists[,1])
num_mindists<-length(mindists[,1])

x<-as.matrix(expand.grid(rep(list(1:(num_majdists*num_mindists)), nvotes))) 
## Combinations of potential slates
possible_lists<-c()
for(line in 1:nrow(x)){
  if(!is.unsorted(x[line,])){
    possible_lists<-rbind(possible_lists,x[line,])
  }
}
combs.num<-nrow(possible_lists)

# Possible type of nominees (given by one maj dist and one min dist)
combs<-possible_lists

save(combs,file=paste0(prefix_tbl,"slate_indices",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))



# Generate index for specific distributions
combs.mat<-matrix(NA,num_majdists*num_mindists,7)
colnames(combs.mat)<-c("index","majdist","mindist","amaj","bmaj","amin","bmin")
combs.mat[,1]<-c(1:(num_majdists*num_mindists))
combs.mat[,2:3]<-as.matrix(expand.grid(c(1:num_majdists),c(1:num_mindists)))
for(i in 1:(num_majdists*num_mindists)){
  combs.mat[i,4]<-majdists[combs.mat[i,2],1]
  combs.mat[i,5]<-majdists[combs.mat[i,2],2]
  combs.mat[i,6]<-mindists[combs.mat[i,3],1]
  combs.mat[i,7]<-mindists[combs.mat[i,3],2]
}

save(combs.mat,file=paste0(prefix_tbl,"slate_indices_to_distribution",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))



##########
## TEST ##
##########

## EQUILIBRIUM : 1 by slate.

# Storing payoff matrix
payoffmats_maj<-array(NA,dim=c(combs.num,numstratmaj,numstratmaj))
payoffmats_min<-array(NA,dim=c(combs.num,numstratmin,numstratmin))

for(slate_index in 1:combs.num){
    
  
  #Matrices to store the results for this slate
  # Minority welfare
  minmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
  rownames(minmatrix)<-minrulesnames
  colnames(minmatrix)<-majrulesnames
  
  # Majority welfare
  majmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
  rownames(majmatrix)<-minrulesnames
  colnames(majmatrix)<-majrulesnames
  
  # Minority blocks
  blockmatrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
  rownames(blockmatrix)<-minrulesnames
  colnames(blockmatrix)<-majrulesnames
  
  # minority blocks per nominee
  blockarray<-array(NA,dim=c(numstratmin,numstratmaj,nvotes))
  rownames(blockarray)<-minrulesnames
  colnames(blockarray)<-majrulesnames
  
  # Share of efficiency achieved
  share_eff_welfarematrix<-matrix(NA,nrow=numstratmin, ncol=numstratmaj)
  rownames(share_eff_welfarematrix)<-minrulesnames
  colnames(share_eff_welfarematrix)<-majrulesnames
  
  #

  
# For one slate of nominee types, draw the intensities:
  slate_types<-combs[slate_index,]
  matrixval_maj<-c()
  matrixval_min<-c()
  for(nomination in 1:nvotes){
    nomin_type<-combs.mat[slate_types[nomination],]
    matrixval_maj<-cbind(matrixval_maj,int_beta3_f(nomin_type[4],nomin_type[5],1,ns,size_maj))
    matrixval_min<-cbind(matrixval_min,int_beta3_f(nomin_type[6],nomin_type[7],1,ns,size_min))
  }
  
  
  # compute outcomes under filibuster and majority rule
  majintens<-tot_intensities(matrixval_maj)
  minintens<-tot_intensities(matrixval_min)
  
  eff_outcome<-sign(majintens-minintens)
  eff_outcome2<-(eff_outcome+1)/2
 
  efficient_welfare<-rowSums(majintens*eff_outcome2)+rowSums(minintens*(1-eff_outcome2))
  maj_vote_welfare<-rowSums(majintens)
  filib_welfare<-rowSums(minintens)
  share_eff_welfare_majvote<-mean((maj_vote_welfare+efficient_welfare)/(2*efficient_welfare)*100)
  share_eff_welfare_filib<-mean((filib_welfare+efficient_welfare)/(2*efficient_welfare)*100)
  
  for (quantile_maj in majrules){
    for(quantile_min in minrules){
     
      majindex<-which(majrules==quantile_maj)
      minindex<-which(minrules==quantile_min)
      
      # Rule used
      rule_maj<-choosen_rule(nvotes,quantile_maj)
      rule_min<-choosen_rule(nvotes,quantile_min)
      majres<-apply_chosen_rule(matrixval_maj,rule=rule_maj)
      minres<-apply_chosen_rule(matrixval_min,rule=rule_min)
      
      maj_votes<-majres$matrixofvotes
      maj_intens<-majres$matrixofintens
      min_votes<-minres$matrixofvotes
      min_intens<-minres$matrixofintens
    
      welf<-welfare_rule(maj_intens,min_intens,maj_votes,min_votes)
      welf<-unlist(lapply(welf,mean))
 
      # save values
      
      # Majority welfare
      majmatrix[minindex,majindex]<-welf[1]
      # Minority welfare
      minmatrix[minindex,majindex]<-welf[2]
      # Share of efficiency achieved
      share_eff_welfarematrix[minindex,majindex]<-welf[5]
      # number of blocks
      blockmatrix[minindex,majindex]<-welf[6]
      # number of blocks for specific nominee
      blockarray[minindex,majindex,]<-welf[7:(6+nvotes)]
      
      
    }
  }
  # share of efficiency with majority voting
  # share of efficiency with filibuster


payoffmats_maj[slate_index,,]<-majmatrix
payoffmats_min[slate_index,,]<-minmatrix
 majpayoffs<-t(majmatrix)
  minpayoffs<-t(minmatrix)
  blockmat<-t(blockmatrix)
  effmat<-t(share_eff_welfarematrix)
  # Find equilibrium

  # Find Nash
  allnasheq<-allnasheq33(majpayoffs,minpayoffs)
  #Output the fully mix nashes
  fullmixnashes<-allnasheq$fullmix
  lfullmix<-length(fullmixnashes[[1]])
  nfullmix<-length(fullmixnashes)
  
  if(lfullmix!=0){
    for(fullmixequil in 1:nfullmix){
      maj_strat<-fullmixnashes[[fullmixequil]][1,]
      min_strat<-fullmixnashes[[fullmixequil]][2,]
      
      # proba of each case
      probapay<-maj_strat%*%t(min_strat)
      
      #payoffs
      maj_pay<-sum(majpayoffs*probapay)
      min_pay<-sum(minpayoffs*probapay)
      
      #blocks
      expectedblocks<-sum(blockmat*probapay)
         
      #blocks by nominee
      expectedblocks_disagg<-c()
      for(nominee in 1:nvotes){
        bmatrix<-t(blockarray[,,nominee])
        expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
      }
      
      
      #efficiency share
      expectedeffshare<-sum(effmat*probapay)
      
      #save
      newequilibrium<-c(slate_index,maj_strat,min_strat,
                        maj_pay,min_pay,expectedblocks,expectedeffshare,
                        share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
      equilibria<-rbind(equilibria,newequilibrium)
    }
  }
  
  # Pure Nash
  purenashes<-allnasheq$pure
  lpure<-length(purenashes[[1]])
  numpure<-length(purenashes)
  if(lpure!=0){
    for(purequil in 1:numpure){
      
      maj_strat<-purenashes[[purequil]][1,]
      min_strat<-purenashes[[purequil]][2,]
      
      # proba of each case
      probapay<-maj_strat%*%t(min_strat)
      
      #payoffs
      maj_pay<-sum(majpayoffs*probapay)
      min_pay<-sum(minpayoffs*probapay)
      
      #blocks
      expectedblocks<-sum(blockmat*probapay)
      
      #blocks by nominee
      expectedblocks_disagg<-c()
      for(nominee in 1:nvotes){
        bmatrix<-t(blockarray[,,nominee])
        expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
      }
      
      
      #efficiency share
      expectedeffshare<-sum(effmat*probapay)
      #save
      newequilibrium<-c(slate_index,maj_strat,min_strat,
                        maj_pay,min_pay,expectedblocks,expectedeffshare,
                        share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
      equilibria<-rbind(equilibria,newequilibrium)
    }
  }
  
  
  # Partial mix nash
  partmixnashes<-allnasheq$partmix
  lpartmix<-length(partmixnashes[[1]])
  numpartmix<-length(partmixnashes)
  if(lpartmix!=0){
    for(partmixequil in 1:numpartmix){
      
      maj_strat<-partmixnashes[[partmixequil]][1,]
      min_strat<-partmixnashes[[partmixequil]][2,]
      
      # proba of each case
      probapay<-maj_strat%*%t(min_strat)
      
      #payoffs
      maj_pay<-sum(majpayoffs*probapay)
      min_pay<-sum(minpayoffs*probapay)
      
      #blocks
      expectedblocks<-sum(blockmat*probapay)
      
      #blocks by nominee
      expectedblocks_disagg<-c()
      for(nominee in 1:nvotes){
        bmatrix<-t(blockarray[,,nominee])
        expectedblocks_disagg<-c(expectedblocks_disagg,sum(bmatrix*probapay))
      }
      
      
      #efficiency share
      expectedeffshare<-sum(effmat*probapay)
      #save
      newequilibrium<-c(slate_index,maj_strat,min_strat,
                        maj_pay,min_pay,expectedblocks,expectedeffshare,
                        share_eff_welfare_majvote,share_eff_welfare_filib,expectedblocks_disagg)
      equilibria<-rbind(equilibria,newequilibrium)
    }
  }
  
}

# Put all of this in a beautiful table
equilibria<-data.frame(equilibria)
names_indiv_block<-c()
for(nominee in 1:nvotes){
  names_indiv_block<-c(names_indiv_block,paste0("block_on_nominee_",nominee))
}
names(equilibria)<-c("slate index",majrulesnames,minrulesnames,
                     "majwelfare","minwelfare","blocks","effshare","effshare_majvoting",
                     "effshare_filib",names_indiv_block)

save(equilibria,file=paste0(prefix_tbl,"equilibria",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))
write.csv(equilibria,file=paste0(prefix_tbl,"equilibria",rules_suff,"agenda",agendaname,"numnomin",nvotes,".csv"))
save(payoffmats_maj,file=paste0(prefix_tbl,"payoffmats_maj",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))
save(payoffmats_min,file=paste0(prefix_tbl,"payoffmats_min",rules_suff,"agenda",agendaname,"numnomin",nvotes,".Rdata"))