
tot_intensities<-function(matrixval){
  party_size<-nrow(matrixval)/ns
  numvotes<-ncol(matrixval)
  # matrix with party intensity total
  matrixofintens<-matrix(NA,nrow=ns,ncol=numvotes)
  
  #generate simulation index
  id<-rep(1:ns,party_size)
  
  # Sum number of votes (for outcome) and intensities (for welfare)
  for (i in 1:nvotes){
    ## tapply(Summary Variable, Group Variable, Function)
    
    # Sum intensities within party on each issue by simulation id
    intensities2<-tapply(matrixval[,i],id,sum)
    
    matrixofintens[,i]<-intensities2
  }
  # MATRIX OF VOTES: sum intensities for one party
  
  out<-matrixofintens
  out
}
# welfare rule: computes welfare
# takes matrices of total intensities and votes
# of the majority and minority
# all of these will be given by apply_choosen_rule

welfare_rule<-function(majintens,minintens,majvotes,minvotes){
  
  # Generating outcomes
  outcome<-sign(majvotes-minvotes+.1) #+.1 to give ties to maj
  eff_outcome<-sign(majintens-minintens)
  
  # Realized welfare
  real_maj_welf<-rowSums(majintens*outcome) # multiply matrices term by term
  real_min_welf<-rowSums(minintens*(-outcome))
  
  #possible welfare
  poss_maj<-rowSums(majintens)
  poss_min<-rowSums(minintens)
  
  # Normalization of realized welfare
  normal_maj_welf<-(real_maj_welf+poss_maj)/(2*poss_maj)*100
  normal_min_welf<-(real_min_welf+poss_min)/(2*poss_min)*100
  
  # Efficiency share (the weird operations are to transform the 1 and -1 for outcomes into 
  # 1 and 0's)
  eff_outcome2<-(eff_outcome+1)/2
  real_outcome2<-(outcome+1)/2
  blocks<-ncol(outcome)-rowSums(real_outcome2)
  blockmat<-1-real_outcome2
  efficient_welfare<-rowSums(majintens*eff_outcome2)+rowSums(minintens*(1-eff_outcome2))
  realized_welfare<-rowSums(majintens*real_outcome2)+rowSums(minintens*(1-real_outcome2))
  share_eff_welfare<-(realized_welfare+efficient_welfare)/(2*efficient_welfare)*100
  
  out<-list(real_maj_welf,real_min_welf,normal_maj_welf,
            normal_min_welf,share_eff_welfare,blocks)
  for(nominee in 1:nvotes){
    l<-length(out)
    out[[l+1]]<-blockmat[,nominee]
  }
  out
}

############## SCRATCH PAD