###################
###### LIBS  ######
###################

library(plyr)

###################
###### GAMES ######
###################
#Hawk dove
# mat_maj<-matrix(c(0,3,15,9),nrow=2)
# mat_min<-matrix(c(0,15,3,9),nrow=2)
# # Nash: pure mismatch, or mixed 2/3, 1/3
# 
# #Hawk dove with weird 3rd strat
# mat_maj<-matrix(c(0,3,-10,15,9,-10,-10,-10,-10),nrow=3)
# mat_min<-matrix(c(0,15,-10,3,9,-10,-10,-10,-10),nrow=3)
# # Nash: pure mismatch, or mixed 2/3, 1/3
# 
# 
# # other
# mat_maj<-matrix(c(7,9,6,6,7,7,7,0,2),nrow=3)
# mat_min<-matrix(c(0,2,9,3,6,1,0,8,8),nrow=3)
# #(no equil pure  strat, one mix nash (44/65,3/65) for row (maj),
# # (2/22,17/22)for col)
# 
# ###################
# ###### TESTS ######
# ###################
# 
# #pure strat
# nashequil(mat_maj,mat_min)
# 
# #full mix
# fullymixnashequil(mat_maj,mat_min)
# 
# #partial mix
# partialmixnashequil(mat_maj,mat_min)
# 
# #all
# allnasheq<-allnasheq33(mat_maj,mat_min)
# allnasheq$pure
# allnasheq$fullmix
# allnasheq$partmix

###################
###### FUNS  ######
###################

# Find pure nash equilibrium in 2-player normal form game
nashequil<-function(mat_maj,mat_min){

  
  num_strat_maj<-nrow(mat_maj)
  num_strat_min<-ncol(mat_min)
 
  list_actions<-c()
  for(i in 1:num_strat_maj){
    list_actions<-c(list_actions,paste("Action",toString(i)))
  }
  
  br_maj<-function(minstrat){
    payoff<-rep(0,num_strat_maj)
    for(j in 1:num_strat_min ){
      payoff<-payoff+minstrat[j]*mat_maj[,j]
    }
    br_init<-rep(0,num_strat_maj)
    br<-c()
    for (numofbr in which(payoff==max(payoff)) ){
      onebr<-br_init
      onebr[numofbr]<-1
      br<-rbind(br,onebr)
    }
    split(br, row(br))
  }
  
  
  br_maj_corresp<-function(listof_minstrat){
    list_init<-lapply(listof_minstrat,br_maj)
    len<-length(list_init)
    listofbr<-list()
    orderofbestresp<-0
    for (count in 1:len){
      for(count2 in 1:length(list_init[[count]])){
        orderofbestresp<-orderofbestresp+1
        listofbr<-c(listofbr,list_init[[count]][count2])
        names(listofbr)[orderofbestresp]<-toString(orderofbestresp)
      }
    }
    listofbr
  }
  
  br_min<-function(majstrat){
    payoff<-rep(0,num_strat_min)
    for(i in 1:num_strat_maj ){
      payoff<-payoff+majstrat[i]*mat_min[i,]
    }
    br_init<-rep(0,num_strat_min)
    br<-c()
    for (numofbr in which(payoff==max(payoff)) ){
      onebr<-br_init
      onebr[numofbr]<-1
      br<-rbind(br,onebr)
    }
    split(br,row(br))
  }
  
  br_min_corresp<-function(listof_majstrat){
    list_init<-lapply(listof_majstrat,br_min)
    len<-length(list_init)
    listofbr<-list()
    orderofbestresp<-0
    for (count in 1:len){
      for(count2 in 1:length(list_init[[count]])){
        orderofbestresp<-orderofbestresp+1
        listofbr<-c(listofbr,list_init[[count]][count2])
        names(listofbr)[orderofbestresp]<-toString(orderofbestresp)
      }
    }
    listofbr
  }
  
  nashequil<-vector("list")
  count<-0
  for (strat in 1:num_strat_maj){
    
    majstrat<-rep(0,num_strat_maj)
    majstrat[strat]<-1
    
    brmin<-br_min(majstrat)
    
    templist<-unique(br_maj_corresp(brmin))
    
    for (x in templist){

      if (identical(x,majstrat)){
        for(y in brmin){
          
          templist2<-br_maj(y)
          for (z in templist2){

            
            if(identical(majstrat,z)){
              equil<-rbind(majstrat,y)
              colnames(equil)<-list_actions
              rownames(equil)<-c("row_player","col_player")
              count<-count+1
              nashequil[[count]]<-equil
              
            }
          }
        }
      }
  
    }

    
  }
  
  if(length(nashequil)!=0){
    nashequil
  }else{
    list(NULL)
  }
  
  
}



# Find equilibrium in 2-player normal form game
fullymixnashequil<-function(mat_maj,mat_min){
  equilexists<-TRUE
  num_strat_maj<-nrow(mat_maj)
  num_strat_min<-ncol(mat_min)
  
  if(num_strat_maj!=num_strat_min){
    return("Error: was expecting square matrix")
  }
  list_actions<-c()
  for(i in 1:num_strat_maj){
    list_actions<-c(list_actions,paste("Action",toString(i)))
  }
  
  # Assume full mixing
  # System to solve for the majority
  temp<-mat_min-rep(mat_min[,1],num_strat_min)
  temp<-matrix(temp[,-1],nrow=num_strat_maj)
  b<--temp[num_strat_maj,]
  multmat<-matrix(t(temp[-num_strat_maj,]),nrow=num_strat_maj-1)
  for(count in 1:ncol(multmat)){
    multmat[,count]<-multmat[,count]+b
  }
  
  
 # if(abs( det(multmat))>.00001){
if(abs( rcond(multmat))>.0000000001){
    majfullmix<-solve(multmat,b)
    majfullmix<-c(majfullmix,1-sum(majfullmix))
  }else{equilexists<-FALSE}
  #for the minority
  mat_majt<-t(mat_maj)
  mat_mint<-t(mat_min)
  temp2<-mat_majt-rep(mat_majt[,1],num_strat_maj)
  temp2<-matrix(temp2[,-1],nrow=num_strat_min)
  b<--temp2[num_strat_min,]
  multmat<-matrix(t(temp2[-num_strat_min,]),nrow=num_strat_min-1)
  for(count in 1:ncol(multmat)){
    multmat[,count]<-multmat[,count]+b
  }
 # if(abs( det(multmat))>.00001){
if(abs( rcond(multmat))>.0000000001){
    minfullmix<-solve(multmat,b)
    minfullmix<-c(minfullmix,1-sum(minfullmix))
  }else{equilexists<-FALSE}
  if(equilexists){
    equil<-rbind(majfullmix,minfullmix)
    colnames(equil)<-list_actions
    rownames(equil)<-c("row_player","col_player")
    out<-list(NULL)
    if(max(equil)<1 & min(equil)>0){
      out<-list(equil)
    }else{return(list(NULL))}
    return(out)
  }else{return(list(NULL))}
}


# Fin a partially mixed equilibrium with players mixing over same 
# number of actions
partialmixnashequil<-function(mat_maj,mat_min){
  num_strat_maj<-nrow(mat_maj)
  num_strat_min<-ncol(mat_min)
  
  list_actions_row<-c()
  for(i in 1:num_strat_maj){
    list_actions_row<-c(list_actions_row,paste("Action",toString(i)))
  }
  list_actions_col<-c()
  for(i in 1:num_strat_min){
    list_actions_col<-c(list_actions_col,paste("Action",toString(i)))
  }
  
  rownames(mat_maj)<-list_actions_row
  colnames(mat_maj)<-list_actions_col
  colnames(mat_min)<-list_actions_col
  rownames(mat_min)<-list_actions_row
  
  
  max_strat_num<-min(num_strat_min,num_strat_maj)
  # Exclude the fully mix if square matrix
  if(num_strat_min==num_strat_maj){
    max_strat_num<-max_strat_num-1
  }
  
  nashequil<-vector("list")
  count<-0
  for(numstrat in 2:max_strat_num){
    
    possibleactions_row<-combn(list_actions_row, numstrat)
    possibleactions_col<-combn(list_actions_col, numstrat)
    numcomb_row<-ncol(possibleactions_row)
    numcomb_col<-ncol(possibleactions_col)
    # We want to look for an equil where the two players mix
    # on 1<x<max_strat_num strategies
    
    for(i in 1:numcomb_row){
      for(j in 1:numcomb_col){
        
        rowplayer_strats<-rownames(mat_maj) %in% possibleactions_row[,i]
        colplayer_strats<-colnames(mat_min) %in% possibleactions_col[,j]
        
        mat_maj_temp<-mat_maj[rowplayer_strats,colplayer_strats]
        mat_min_temp<-mat_min[rowplayer_strats,colplayer_strats]
        mat_maj_temp
        mat_min_temp
        # candidate equil
        notfullmix<-fullymixnashequil(mat_maj_temp,mat_min_temp)
        notfullmix
        if(length(notfullmix[[1]])>0){
          
          fullprobvecmin<-rep(0,num_strat_min)
          fullprobvecmaj<-rep(0,num_strat_maj)
          fullprobvecmin[colplayer_strats]<-notfullmix[[1]][2,]
          fullprobvecmaj[rowplayer_strats]<-notfullmix[[1]][1,]
          
          #majpayoffs
          
          majpay<-mat_maj%*%fullprobvecmin
          
          #minpayoffs
          minpay<-t(mat_min)%*%fullprobvecmaj
          
          #Is it an equil?
          alsoeq<-0
          if(max(majpay[rowplayer_strats])==max(majpay) & max(minpay[colplayer_strats])==max(minpay) ){
            alsoeq<-1
          }
          alsoeq
          if(alsoeq==1){
            if(num_strat_maj!=num_strat_min){
              if(num_strat_maj<num_strat_min){
                vtemp<-rep(NA,num_strat_min-num_strat_maj)
                fullprobvecmaj<-c(fullprobvecmaj,vtemp)
                list_actions<-list_actions_col
              }else{
                vtemp<-rep(NA,num_strat_maj-num_strat_min)
                fullprobvecmin<-c(fullprobvecmin,vtemp)
                list_actions<-list_actions_row
              }
            }
            
            equil<-rbind(fullprobvecmaj,fullprobvecmin)
            colnames(equil)<-list_actions_col
            rownames(equil)<-c("row_player","col_player")
            count<-count+1
            nashequil[[count]]<-equil
            
          }
        }
      }
    }
  }
  if(length(nashequil)!=0){
    nashequil
  }else{
    list(NULL)
  }
}


allnasheq33<-function(mat_maj,mat_min){
  if(nrow(mat_maj)==ncol(mat_maj)){
    fullymixnash<-fullymixnashequil(mat_maj,mat_min)
  }else{
    fullymixnash<-list(NULL)
  }
  puremixnash<-nashequil(mat_maj,mat_min)
  if(nrow(mat_maj)>2 | ncol(mat_maj)>2){
    partialmixnash<-partialmixnashequil(mat_maj,mat_min)
  }else{partialmixnash<-list(NULL)}
  out<-list(puremixnash,fullymixnash,partialmixnash)
  names(out)<-c("pure","fullmix","partmix")
  out
}
